/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.*;
import java.awt.geom.*;
import java.lang.reflect.*;

/**
 * Dies ist die Oberklasse von XYPanel und ABPanel, den Darstelllungen der
 * beiden Koordinatensysteme. CoordinatePanel implemnetiert diverse gemeinsame
 * Funktionalitaeten.
 */
public abstract class CoordinatePanel extends SimpleDrawablePanel {
/**
 * erste Koordinate der Spitze des Roboterarms
 */
   public int pos1;
/**
 * zweite Koordinate der Spitze des Roboterarms
 */
   public int  pos2;
   public int  next1;public int next2;

   public int currentX, currentY;

   protected static int xC;protected static int yC;protected static int radiusC;protected static int oWidthXY;protected static int oWidthAB;protected static int oHeightXY;protected static int oHeightAB;

   protected static Color pathColor = Color.red;protected static Color goalColor= Color.red;protected static Color nextGoalColor = Color.gray;
   protected static Color forbiddenColor=new Color(225,225,225);protected static Color obstacleColor=new Color(155,155,155);protected static Color shadowColor=new Color(210, 210, 210);

/**
 * Instanz von ObstacleModule uebernimmt Verwaltung und Berechnung der Hindernisse.
 */
   protected static ObstacleModule obm=new ObstacleModule(120,270,200,225);

   public GeneralPath goals=new GeneralPath();public GeneralPath path=new GeneralPath();
   public static boolean drawPath;public static boolean drawGoals;public static boolean drawNextGoal=true;public static boolean drawCircle;

   protected static Area forbiddenAB=new Area();protected static Area forbiddenXY=new Area();protected static Area allowedAB=new Area();protected static Area allowedXY=new Area();
   protected static Area shadowsAB=new Area();protected static Area shadowsXY=new Area();protected static Area obstaclesXY=new Area();

   protected static Stroke basicStroke=new BasicStroke();protected static Stroke goalStroke=new BasicStroke();

/**
 * Diese Methode loescht alle Hindernisse; wird aufgerufen, wenn der Reset-Button
 * gedrueckt wird
 */
   public static void deleteObstacles() {
      obm=new ObstacleModule(120,270,200,225);
      shadowsAB.reset();
      shadowsXY.reset();
      obstaclesXY.reset();
      BasicHandler.obstacleMode=false;
   }

/**
  * Fuegt ein neues Hindernis zu den im ObstacleModule verwalteten Hindernisse hinzu
  */
   public static void addAsObstacle() {
      double x=xC-oWidthXY, y=oHeightXY-yC;
      double alphaC=Math.acos(x/Math.sqrt(x*x+y*y));
      if(y<0) alphaC*=-1;

      obm.addAsObstacle(x, y, radiusC, AnimationLoop.data.getAlpha()>alphaC);

      shadowsXY.add(new Area(obm.shadowsXY));

      Area a=new Area(obm.shadowsAB);
      a.subtract(forbiddenAB);
      shadowsAB.add(a);

      obstaclesXY=new Area(obm.obstacles);
   }

/**
 * Mittels dieser Methode werden die Daten des Kreises, der waehrend der Eingabe
 * eines Hinderisses zu sehen ist, aktualisiert.
 * @param   x   x-Koordinate Kreismittelpunkt
 * @param   y   y-Koordinate Kreismittelpunkt
 * @param   r   Radius
 */
   public static void setCircle(int x, int y , int r) {
      xC=x;
      yC=y;
      radiusC=r;
   }

   public CoordinatePanel(DataModel data, int w, int h, String t) {
      super(data, w,h, t);
   }

/**
 * Methode verifiziert, ob ein Punkt mit Koordinaten (x,y) erreichbar ist
 */
   public abstract boolean isPointAllowed(double x, double y);

/**
 * Methode verifiziert, ob sich der Roboterarm von einem Punkt mit Koordinaten
 * (x1,y1) auf einer Gerade zu einem Punkt mit Koordinaten (x2,y2) bewegen kann.
 */
   public abstract boolean isLineAllowed(double x1, double y1, double x2, double y2);

   public abstract void updateDataModel(int a, int b);

   public void initGoals() {
      initGeneralPath(goals);
      next1=pos1;
      next2=pos2;
   }

   protected void initGeneralPath(GeneralPath p) {
      p.reset();
      p.moveTo(pos1, pos2);
   }

/**
 * Zeichen des Pfades, d.h. des Weges, den der Roboterarm schon abgelaufen hat.
 */
   protected void drawPath(Graphics2D g2) {
      g2.setStroke(pathStroke);
      g2.setPaint(pathColor);
      g2.draw(path);

      Point2D p=path.getCurrentPoint();
      if (p != null)
        g2.drawLine((int)p.getX(),(int)p.getY(),pos1, pos2);
   }

/**
 * Zeichen des Kantenzuges, den der Roboterarm noch ablaufen muss.
 */
   public void drawGoals(Graphics2D g2) {
      g2.setPaint(goalColor);
      g2.setStroke(goalStroke);
      g2.draw(goals);

      if (drawNextGoal) {
         g2.setColor(nextGoalColor);
         Point2D p=goals.getCurrentPoint();
         g2.drawLine((int)p.getX(),(int)p.getY(),next1, next2);
      }
   }

   public void drawCurrentCoordinates(Graphics2D g) {};

   public double getXCoordinate(int s, int t) { return s;}
   public double getYCoordinate(int s, int t) { return t;}

   public double getACoordinate(int s, int t) { return s;}
   public double getBCoordinate(int s, int t) { return t;}

/**
 * Hinzufuegen eines Zieles zu der Zielliste in der Klasse AnimationLoop. Diese Methode
 * benutzt Reflection, um die richtige Methode im AnimationLoop (addGoalXY oder
 * addGoalAB) aufzurufen
 * @param  s  erste Koordinate
 * @param  t  zweite Koordinate
 */
   public void addGoal(double s, double t) {
      goals.lineTo(offsetWidth+(float)s, offsetHeight-(float)t);

      try {
         Class[] array3={double.class, double.class};

         String className=RoboArmFactory.getClassSubString(getClass().getName());
         Method m=AnimationLoop.class.getMethod("addGoal"+className.substring(0,2), array3);

         Double [] d={new Double(s),new Double(t)};
         m.invoke(null, d);
      } catch(Exception e) {System.out.println("..."+e.toString());}
   }
}