/** 
 * This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
 
package ch.educeth.roboarm; 
 
import java.awt.*;
import java.awt.event.*;
 
 /** 
  * Diese Klasse ist der Event-Handler, der installiert ist, falls kein Pfad eingegeben 
  * wird. Moegliche Events sind: Click in ein Fenster und damit Start der Animation, 
  * 'Drag' des Roboterarms, Einzeichnen von Hindernissen (nur XYPanel).
  */
public class NormalHandler extends BasicHandler {
   private boolean dragEnabled, obstacleEnabled, possiblyObstacleEnabled;
   private int xC, yC;
   private long time;

   public void mouseClicked(MouseEvent e) {
      info.setText(null);
      CoordinatePanel c=(CoordinatePanel) e.getSource();

      if(!dragEnabled && !obstacleEnabled && (Math.abs(time-System.currentTimeMillis())>200)) {
         int s=e.getX()-c.offsetWidth, t=c.offsetHeight-e.getY();  

         double newX=c.getXCoordinate(s,t);
         double newY=c.getYCoordinate(s,t);        
         double newA=c.getACoordinate(s,t);
         double newB=c.getBCoordinate(s,t);
         
         boolean b1=c.isPointAllowed(c.offsetWidth+s, c.offsetHeight-t);
         boolean b2=isAllowed(newX, newY, newA, newB);
         
         if (b1 && b2) {
            BasicHandler.changeButtons(false); 
            AnimationLoop.stopAnimation(); 
            CoordinatePanel.drawGoals=false; 
            CoordinatePanel.drawPath=true;       
            addGoals(newX, newY, newA, newB); 
            AnimationLoop.startAnimation();         
         } else info.setText("Dieser Punkt ist nicht erreichbar");
      }   
      
      time=System.currentTimeMillis();       
   }
 
   private void update(MouseEvent e) { 
      CoordinatePanel c=(CoordinatePanel) e.getSource();  
      int s=e.getX()-c.offsetWidth, t=c.offsetHeight-e.getY();

      cXY.counter=0; cAB.counter=0;

      if(Math.sqrt(s*s+t*t)>=AnimationLoop.data.getL1()+AnimationLoop.data.getL2()) {s-=1; t-=1;}   

      c.updateDataModel(s, t);   
   } 
    
   public void mousePressed(MouseEvent e){     
      info.setText(null);   
      int x=e.getX(), y=e.getY();
   
      CoordinatePanel c=(CoordinatePanel) e.getSource();

      boolean b1=Math.abs(x-c.pos1)<4 && Math.abs(y-c.pos2)<4 && c.isPointAllowed(x, y);
      boolean b2=((e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) && (!b1) && (c == cXY) && ((XYPanel)cXY).isCenterValid(x,y) && !((XYPanel)cXY).isRobotArmContained(x, y, 4);

      if(!AnimationLoop.isRunning() && b1) { 
         dragEnabled=true;
         CoordinatePanel.drawPath=false;
         SimpleDrawablePanel.doNotDraw=true;
         update(e);
      } else if((!AnimationLoop.isRunning()) && b2) {
         possiblyObstacleEnabled=true;
         CoordinatePanel.drawPath=false;
         CoordinatePanel.drawGoals=false;                             
         xC=x; yC=y;       
      }     
   };
      
   public void mouseDragged(MouseEvent e) {
      int x=e.getX(), y=e.getY(); 

      if (dragEnabled && ((CoordinatePanel) e.getSource()).isPointAllowed(x,y)) update(e);
      else if((e.getSource()==cXY) && (obstacleEnabled || possiblyObstacleEnabled)) {
         if(possiblyObstacleEnabled) {            
            CoordinatePanel.drawCircle=true;
            CoordinatePanel.setCircle(xC,yC,5);
            CoordinatePanel.drawCircle=true;
            CoordinatePanel.drawPath=false;
            CoordinatePanel.drawGoals=false;                 
            possiblyObstacleEnabled=false;
            obstacleEnabled=true;  
         }      
      
         int r=(int)Math.sqrt((x-xC)*(x-xC)+(y-yC)*(y-yC)); 
      
         boolean b1=!((XYPanel)cXY).isRobotArmContained(xC, yC, r);
         boolean b2=r<100; 
            
         if (b1 && b2) {
            CoordinatePanel.radiusC=r;
            cXY.counter=0;
            cXY.update(AnimationLoop.data, null);
         } else if (!b1) info.setText("Der Roboterarm darf vom Hindernis nicht verdeckt werden");  
         else info.setText("Der Kreis ist zu gross...");
      }
   }
   
   public void mouseReleased(MouseEvent e) {   
      if(dragEnabled) {
         dragEnabled=false; 
         SimpleDrawablePanel.doNotDraw=false;
         CoordinatePanel.drawPath=true;
      } else if(obstacleEnabled) {
         obstacleEnabled=false;
         possiblyObstacleEnabled=false;
         
         BasicHandler.obstacleMode=true;
         BasicHandler.resetAnimation();

         CoordinatePanel.addAsObstacle();
         CoordinatePanel.drawCircle=false;
         CoordinatePanel.drawPath=true;
         CoordinatePanel.drawGoals=false;      
         xC=-100; yC=-100;

         cXY.counter=0;
         cXY.update(AnimationLoop.data, new Message(true,true , false));
         cAB.counter=0;
         cAB.update(AnimationLoop.data, new Message(true,true , false));           
      }
   }
   
   public void mouseExited(MouseEvent e) {dragEnabled=false;}
   
   public void mouseMoved(MouseEvent e) {
      int x=e.getX(), y=e.getY();
      CoordinatePanel c=(CoordinatePanel) e.getSource();

      c.currentX=x;
      c.currentY=y;

      c.drawCurrentCoordinates((Graphics2D)c.getGraphics());
    
      if(c.isPointAllowed(x, y)) c.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
      else c.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));    
   };  
}
