/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Diese Klasse ist das Frame, das nach dem Klick auf den Applet-Button geoeffnet
 * wird. Im Konstruktor werden jegliche Initialisierungen vorgenommen.
 */

public class RobotFrame extends JFrame {
   private Box box1=Box.createHorizontalBox();private Box box2=Box.createHorizontalBox();

   private Color background=new Color(220,220,220);

   private ButtonGroup group=new ButtonGroup();
   private JRadioButton lineAB;private JRadioButton seqAB;private JRadioButton lineXY;private JRadioButton seqXY;
   private JSlider speed= new JSlider(SwingConstants.HORIZONTAL,1,9,5);
   private JToggleButton path=new JToggleButton("Start");
   private JButton startButton=new JButton("Start");public JButton reset=new JButton("Alles Zuruecksetzen");

   private JTextField info=new JTextField();

   private JPanel panel1 = new JPanel();private JPanel panel2 = new JPanel();private JPanel panel3 = new JPanel(new GridLayout(0,1));private JPanel panel4 = new JPanel();
   private JPanel bigPanel= new JPanel();private JPanel panelX = new JPanel(new BorderLayout());private JPanel auswahl=new JPanel(new BorderLayout());

   private DataModel data=new DataModel(Math.PI/2,Math.PI/2,120,70);

   private Status stat=new Status(data, background);

   private XYPanel cXY;
   private ABPanel cAB;
   private TPanel cT;

   public RobotFrame() {
     super("Roboterarm-Applet");

// Info-Label
      info.setPreferredSize(new Dimension(400,40));
      info.setBackground(background);
      info.setBorder(BorderFactory.createTitledBorder("Information"));
      info.setEditable(false);

// Pfadauswahl
      path.setBackground(background);

      panel4.setBackground(background);
      panel4.setBorder(BorderFactory.createTitledBorder("Pfad vorgeben"));
      panel4.setPreferredSize(new Dimension(200,67));
      panel4.add(path);

      path.setSelected(false);

      path.addActionListener(new ActionListener () {
         public void actionPerformed(ActionEvent e) {

            if(!path.isSelected()) {
              BasicHandler.createHandlerInstances(RoboArmFactory.getPackageSubString()+"NormalHandler");
              path.setSelected(false);
              path.setText("Start");
              CoordinatePanel.drawPath=true;

              if(AnimationLoop.isAB) lineAB.setSelected(true); else lineXY.setSelected(true);
              AnimationLoop.isLine=true;          
              AnimationLoop.startAnimation();
            } else {
               BasicHandler.changeButtons(false);
               path.setText("Stop");
               AnimationLoop.stopAnimation();
               BasicHandler.createHandlerInstances(RoboArmFactory.getPackageSubString()+"PathHandler");
            }
         }
      });

// Reset-Button
      reset.setBackground(background);
      reset.addActionListener(new ActionListener () {
         public void actionPerformed(ActionEvent e) {
            CoordinatePanel.deleteObstacles();

            seqAB.setEnabled(true);
            seqXY.setEnabled(true);

            AnimationLoop.data.setAngles(Math.PI/2, Math.PI/2);

            cXY.update(AnimationLoop.data, new Message(true, true, false));
            cAB.update(AnimationLoop.data, new Message(true, true, false));
            cT.update(AnimationLoop.data, new Message(true, false, false));
            stat.update(AnimationLoop.data, new Message(true,false,false));
         }
      });

// Bewegungsauswahl initialisieren
      lineAB=new JRadioButton("Linie in Alpha/Beta-Koord.",true);
      lineAB.setActionCommand("lAB");
      lineAB.setBackground(background);

      seqAB=new JRadioButton("Sequentiell Alpha/Beta-Koord.");
      seqAB.setActionCommand("sAB");
      seqAB.setBackground(background);

      lineXY=new JRadioButton("Linie in X/Y-Koord.");
      lineXY.setActionCommand("lXY");
      lineXY.setBackground(background);

      seqXY=new JRadioButton("Sequentiell X/Y-Koord.");
      seqXY.setActionCommand("sXY");
      seqXY.setBackground(background);

      group.add(lineAB);
      group.add(seqAB);
      group.add(lineXY);
      group.add(seqXY);

      RadioListener listener = new RadioListener();
      lineAB.addActionListener(listener);
      seqAB.addActionListener(listener);
      lineXY.addActionListener(listener);
      seqXY.addActionListener(listener);

// Geschwindigkeitsslider initialisieren
      speed.setPreferredSize(new Dimension(200,67));
      speed.setBackground(background);
      speed.setMajorTickSpacing(1);
      speed.setPaintTicks(true);
      speed.setPaintLabels(true);
      speed.setBorder(BorderFactory.createTitledBorder("Animationsgeschw."));

      speed.addChangeListener(new ChangeListener () {
         public void stateChanged(ChangeEvent e) {
            AnimationLoop.adjustSpeed(((JSlider)e.getSource()).getValue());
         }
      });

// Panels initialisieren
      panel3.setBackground(background);
      panel3.setBorder(BorderFactory.createTitledBorder("Bewegung"));
      panel3.add(lineAB);
      panel3.add(seqAB);
      panel3.add(lineXY);
      panel3.add(seqXY);
      panel3.setMaximumSize(new Dimension(200,130));

      panel2.setBackground(background);
      panel2.add(panel4);
      panel2.add(speed);
      panel2.add(reset);

      panel1.setBackground(background);
      panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
      panel1.add(stat);
      panel1.add(panel3);

// Panel & Co initialisieren
      AnimationLoop.data=data;
      AnimationLoop.createSingleInstance(RoboArmFactory.getPackageSubString()+"AnimationLineAB");

      cXY=new XYPanel(data,400,300);
      cAB=new ABPanel(data, 350,300);
      cAB.initGoals();
      cXY.initGoals();
      cT=new TPanel(data, 350,300);

      BasicHandler.info=info;
      BasicHandler.reset=reset;
      BasicHandler.lineAB=lineAB; BasicHandler.seqAB=seqAB;
      BasicHandler.lineXY=lineXY; BasicHandler.seqXY=seqXY;
      BasicHandler.group=group;
      BasicHandler.setPanel(cXY, cAB);
      BasicHandler.createHandlerInstances(RoboArmFactory.getPackageSubString()+"NormalHandler");

// Objekte zum Applet hinzufuegen
      box1.add(cXY);
      box1.add(Box.createHorizontalStrut(5));
      box1.add(cAB);

      auswahl.setMinimumSize(new Dimension(400,250));
      auswahl.setMaximumSize(new Dimension(400,250));
      auswahl.setPreferredSize(new Dimension(400,250));
      auswahl.setLayout(new GridLayout(1,0));
      auswahl.add(panel1);
      auswahl.add(panel2);

      panelX.setBackground(background);
      panelX.setMinimumSize(new Dimension(400,300));
      panelX.setMaximumSize(new Dimension(400,300));
      panelX.setPreferredSize(new Dimension(400,300));
      panelX.add(info, BorderLayout.NORTH);
      panelX.add(auswahl, BorderLayout.SOUTH);

      box2.add(panelX);
      box2.add(Box.createHorizontalStrut(5));
      box2.add(cT);

      bigPanel.setPreferredSize(new Dimension(755,635));
      bigPanel.add(box1);
      bigPanel.add(box2);

      getContentPane().add(bigPanel, BorderLayout.NORTH);
      pack();
   }

/**
 * Listener Klasse fuer die Bewegungsauswahl.
 */
   class RadioListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
         AnimationLoop.isLine=e.getActionCommand().equals("lXY") || e.getActionCommand().equals("lAB");
         AnimationLoop.isAB=e.getActionCommand().substring(1,3).equals("AB");
         AnimationLoop.createSingleInstance(RoboArmFactory.getPackageSubString()+"AnimationLine"+e.getActionCommand().substring(1,3));
      }
   }
}