/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.geom.Line2D;

/**
 * Die Klasse SimpleChart zeichnet ein simples zweidimensionales Koordinatensystem
 */
public class SimpleChart {
   private double majorSX;private double majorSY;private double minorSX;private double minorSY;
   private double width;private double height;

   private int maxX;private int maxY;private int minX;private int minY;
   private int[] origin= new int[2];

   private Color majorColor=new Color(215,215,215);
   private Color minorColor=new Color(235,235,235);
   private Color axesColor=Color.black;

   private Font font=new Font("Arial", Font.PLAIN, 10);
   private FontRenderContext frc;
   private Line2D.Double l=new Line2D.Double();

/**
  * @param   r    Bereich, in den das Koordinatensystem zu liegen kommt; die Beschriftung der Achsen wird
  *               wird knapp ausserhalb dieses Bereichs liegen
  * @param   maX  maximaler Wert der x-Achse
  * @param   miX  minimaler Wert der x-Achse
  * @param   maY  maximaler Wert der y-Achse
  * @param   miX  minimaler Wert der y-Achse
  * @param   majX gibt an, nach wievielen x-Werten eine beschriftete, 'starke' Linie (vertikal) gezeichnet werden soll
  *                   (z.B. x-Achse mit Bereich von 0 bis 100, alle 10 Einheiten beschriftete Linie: majX=10)
  * @param   majY gibt an, nach wievielen y-Werten eine beschriftete, 'starke' Linie (horizontal) gezeichnet werden soll
  * @param   zX   Anzahl der 'schwachen' Linien zwischen zwei starken Linien auf der x-Achse.
  * @param   zY   Anzahl der 'schwachen' Linien zwischen zwei starken Linien auf der y-Achse.
  */
   public SimpleChart(Rectangle r, int maX, int miX, int maY, int miY, int majX, int majY, int zX, int zY) {
      maxX=maX;
      minX=miX;
      maxY=maY;
      minY=miY;

      majorSX=majX;
      majorSY=majY;

      minorSX=zX+1;
      minorSY=zY+1;

      width=r.width;
      height=r.height;
      origin[0]=r.x;
      origin[1]=(int)(r.y+r.height);
   }

/**
 * Diese Methode soll in der Paint-Methode der Klienten-Klasse aufgerufen werden, um den Chart
 * zu zeichnen
 */
   public void drawGrid(Graphics2D g) {
      g.setFont(font);
      frc=g.getFontRenderContext();

      drawGrid(g, (int)(width*majorSX/(maxX-minX)), (int) majorSX, minX, 0, 0,0,0, (int)height, (int)minorSX, 0, 0, 5);
      drawGrid(g, 0,0,0, (int)width,(int)(height*majorSY/(maxY-minY)), (int) majorSY, minY, 0, 0, (int) minorSY, 5, 0);
      drawAxes(g);
   }

   private void drawGrid(Graphics2D g, int dX, int valueX,int mX, int wX, int dY, int valueY,
                         int mY, int wY, int miX, int miY, int axesX, int axesY) {

      int currentValue=mX+mY;
      Rectangle2D r;

      g.setColor(axesColor);
      r=font.getStringBounds(Integer.toString(currentValue), frc);

      if (dX!=0) g.drawString(Integer.toString(currentValue), (int)(origin[0]-r.getWidth()/2), origin[1]+15);
      else g.drawString(Integer.toString(currentValue), (int)(origin[0]-r.getWidth()-10), origin[1]+2);

      for(int i=origin[0]+dX, j=origin[1]-dY; i<=(int)(origin[0]+width) && j>=(int)(origin[1]-height); i+=dX, j-=dY) {
         currentValue+=valueX+valueY;

         g.setColor(minorColor);
         drawMinorLines(g, i-dX, dX, valueX, wX, j+dY, dY, valueY, wY);

         g.setColor(majorColor);
         g.drawLine(i, j, i+wX, j-wY);

         g.setColor(axesColor);
         g.drawLine(i, j, i-axesX, j+axesY);

         r=font.getStringBounds(Integer.toString(currentValue), frc);
         if (dX!=0) g.drawString(Integer.toString(currentValue), (int)(i-r.getWidth()/2), origin[1]+15);
         else g.drawString(Integer.toString(currentValue), (int)(origin[0]-r.getWidth()-10), j+2);
      }
   }

   private void drawMinorLines(Graphics2D g, int sX, int dX, int mSX, int wX, int sY, int dY, int mSY, int wY) {
      double pixelsX=width*(mSX/minorSX)/(maxX-minX);
      double pixelsY=height*(mSY/minorSY)/(maxY-minY);

      for(int i=1, j=1; i<minorSX && j<minorSY; i++, j++) {
         l.setLine(sX+i*pixelsX, sY-j*pixelsY, sX+i*pixelsX+wX, sY-j*pixelsY-wY);
         g.draw(l);
      }
   }

   private void drawAxes(Graphics g) {
      g.setColor(axesColor);
      g.drawLine(origin[0]-5, origin[1], (int)(origin[0]+width), origin[1]);
      g.drawLine(origin[0], origin[1]+5, origin[0], (int)(origin[1]-height));
   }

   public void setXSpacing(int major, int minor) {
      majorSX=major;
      minorSX=minor;
   }

   public void setYSpacing(int major, int minor) {
      majorSY=major;
      minorSY=minor;
   }

/** Neueinstellung der Range der x-Achse
  * @param   maX  maximaler Wert der x-Achse
  * @param   miX  minimaler Wert der x-Achse
  */
   public void setNewBoundsX(int max, int min) {
      maxX=max;
      minX=min;
   }

/** Neueinstellung der Range der y-Achse
  * @param   max  maximaler Wert der y-Achse
  * @param   min  minimaler Wert der y-Achse
  */
   public void setNewBoundsY(int max, int min) {
      maxY=max;
      minY=min;
   }

/**
  * @param  maC   Farbe der 'dicken' Linien
  * @param  miC   Farbe der 'duennen' Linien
  * @param  aC    Farbe der beiden Achsen
  */
   public void setColors(Color maC, Color miC, Color aC) {
      majorColor=maC;
      minorColor=miC;
      axesColor=aC;
   }
}
