/* This file is part of Roboarm.
 *
 * Roboarm is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Roboarm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Roboarm; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package ch.educeth.roboarm;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.Arc2D;

/**
 * XYPanel implementiert die Darstellung des (x,y)-Koordinatenraumes.
 */
public class XYPanel extends CoordinatePanel {
/**
 * Koordinaten des Gelenks ('Ellbogen') des Roboterarms.
 */
   public int[] one=new int[2];
   private Area ellipse;
   private Shape s;
   private boolean drawArcs=true;
   private Arc2D.Double alphaArc=new Arc2D.Double(Arc2D.OPEN);private Arc2D.Double betaArc=new Arc2D.Double(Arc2D.OPEN);

   private Color blockColor=new Color(190,190,190);
   private Color robotColor=Color.black;

   protected Stroke robotStroke=new BasicStroke(4.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
   protected Stroke arcStroke=new BasicStroke(1.5f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
   private Stroke auxStroke=new BasicStroke(2.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);

   public XYPanel(DataModel data, int w, int h) {
      super(data, w, h,"X-Y-Koordinaten");

      oWidthXY=w/2; oHeightXY=h-75;
      offsetWidth=w/2; offsetHeight=h-75;

      personalDelay=0.5;

      one[0]=(int) (oWidthXY+data.getL1()*Math.cos(data.getAlpha()));
      one[1]=(int) (oHeightXY-data.getL1()*Math.sin(data.getAlpha()));
      pos1=(int) (one[0]-data.getL2()*Math.cos(data.getAlpha()+data.getBeta()));
      pos2=(int) (one[1]+data.getL2()*Math.sin(data.getAlpha()+data.getBeta()));

      s=new Rectangle2D.Double(0,oHeightXY ,oWidthXY+35,75);

      ellipse=new Area(new Ellipse2D.Double(oWidthXY+data.getL2()-data.getL1()-1, oHeightXY+data.getL2()-data.getL1()-1, 2*(data.getL1()-data.getL2())+2,2*(data.getL1()-data.getL2())+2));
      allowedXY=new Area(new Ellipse2D.Double(oWidthXY-data.getL1()-data.getL2(), oHeightXY-data.getL1()-data.getL2(), 2*(data.getL1()+data.getL2()),2*(data.getL1()+data.getL2())));
      allowedXY.subtract(new Area(new Rectangle2D.Double(0,oHeightXY,w,h-oHeightXY)));
      allowedXY.subtract(new Area(new Ellipse2D.Double(oWidthXY+data.getL2()-data.getL1(), oHeightXY+data.getL2()-data.getL1(), 2*(data.getL1()-data.getL2()),2*(data.getL1()-data.getL2()))));
      allowedXY.subtract(new Area(new Ellipse2D.Double(oWidthXY-data.getL1()-data.getL2(), oHeightXY-data.getL2(), 2*data.getL2(),2*data.getL2())));
      allowedXY.add(new Area(new Ellipse2D.Double(oWidthXY+data.getL1()-data.getL2(), oHeightXY-data.getL2(), 2*data.getL2(),2*data.getL2())));
      forbiddenXY=new Area(new Rectangle2D.Double(0,0,w,h));
      forbiddenXY.subtract(allowedXY);

      initGeneralPath(goals);

      f=new Font("Arial", Font.BOLD, 14);
   }

/**
 * Mit dieser Methode wird getestet, ob ein neues Hindernis, das gerade eingegeben wird, auf den
 * Roboterarm zu liegen kommt
 */
   public boolean isRobotArmContained(int x, int y,int r) {
      GeneralPath p=new GeneralPath();

      p.moveTo(oWidthXY-2, oHeightXY);
      p.lineTo(one[0]-2, one[1]-2);
      p.lineTo(pos1+2, pos2-2);
      p.lineTo(pos1+2, pos2+2);
      p.lineTo(one[0]+2, one[1]+2);
      p.lineTo(oWidthXY+2, oHeightXY);

      Area a=new Area(p);
      a.intersect(new Area(new Ellipse2D.Double(x-r, y-r, 2*r,2*r)));
      return !a.isEmpty();
   }

   public boolean isPointAllowed(double x, double y) {
      return !(forbiddenXY.contains(x,y) || shadowsXY.contains(x,y));
   }

   public boolean isLineAllowed(double x1, double y1, double x2, double y2) {
      GeneralPath p=new GeneralPath();
      p.moveTo((float)x1, (float) y1);
      p.lineTo((float)x2, (float)y2);
      p.lineTo((float)x2, (float)(y2-0.1));
      p.lineTo((float)x1, (float)(y1-0.1));
      p.lineTo((float)x1, (float)y1);

      Area ar=new Area(p);
      Area a=new Area(forbiddenXY); a.intersect(ar);
      Area b=new Area(ellipse); b.intersect(ar);
      Area c=new Area(shadowsXY); c.intersect(ar);

      return a.isEmpty() && b.isEmpty() && c.isEmpty();
   }

/**
 * Methode testet, ob das Zentrum eines neu eingegebenen Hindernisses im erreichbaren Bereich liegt
 */
   public boolean isCenterValid(int x, int y) {
      return !forbiddenXY.contains(x,y);
   }

/**
 * Methode zeichnet die Koordinaten (x,y), wo sich der Cursor gerade befindet (Orientierungshilfe,
 * da im XYPanel kein Koordinatensystem eingezeichnet wird
 */
   public void drawCurrentCoordinates(Graphics2D g2D) {
      g2D.setFont(f);
      g2D.setPaint(blockColor);
      g2D.fillRect(10, oHeightXY+40, 100, 25);
      g2D.setPaint(Color.white);
      g2D.setStroke(auxStroke);
      g2D.drawRect(15, oHeightXY+19, 95, 41);
      g2D.setStroke(arcStroke);
      g2D.drawString("Cursor:", 20, oHeightXY+34);
      g2D.drawString("("+(currentX-oWidthXY)+","+(oHeightXY-currentY)+")", 20, oHeightXY+54);
   }

   public void paintComponent(Graphics g) {
      Graphics2D g2=(Graphics2D) g;

      super.paintComponent(g2);

      g2.setPaint(shadowColor);
      g2.fill(shadowsXY);

      g2.setPaint(obstacleColor);
      g2.fill(obstaclesXY);
      if (drawCircle) g.fillOval(xC-radiusC, yC-radiusC, 2*radiusC, 2*radiusC);

      g2.setPaint(forbiddenColor);
      g2.fill(forbiddenXY);

      if (drawGoals) drawGoals(g2);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      if (drawPath) drawPath(g2);
      if (drawArcs) drawArcs(g2);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);

      g2.setPaint(blockColor);
      g2.fill(s);

      g2.setPaint(robotColor);
      g2.setStroke(robotStroke);
      g2.drawLine(oWidthXY, oHeightXY, one[0], one[1]);
      g2.drawLine(one[0], one[1], pos1, pos2);

      drawCurrentCoordinates(g2);
   }

/**
 * Diese Methode zeichnet die beiden Winkelboegen (nur sichtbar falls keine Animation laeuft
 */
   public void drawArcs(Graphics2D g2) {
      DataModel data=AnimationLoop.data;
      int alpha, beta, offAlpha;

      alpha=(int)Math.toDegrees(data.getAlpha());
      beta=(int)Math.toDegrees(data.getBeta());

      if (alpha<45) offAlpha=0; else offAlpha=9;

      g2.setStroke(arcStroke);
      g2.setPaint(Color.blue);
      g2.drawArc(oWidthXY-30, oHeightXY-30, 60,60,0,alpha-offAlpha);
      g2.setPaint(Color.magenta);

      int delta=-(int)Math.toDegrees(Math.asin((pos2-one[1])/data.getL2()));

      if (pos1-one[0]<0) if (delta<0) delta=-delta-180; else delta=180-delta;

      if (delta>0) {
         g2.drawArc(one[0]-30, one[1]-30, 60,60,0,delta);
         g2.drawArc(one[0]-30, one[1]-30, 60,60,360+delta-beta,beta);
      } else g2.drawArc(one[0]-30, one[1]-30, 60,60,360-beta+delta,beta);
   }

   public void update(Observable o, Object i) {
      DataModel data=(DataModel) o;
      Message msg=(Message)i;

      if (msg!=null) {
         counter=0;
         drawArcs=msg.endOfAnimation;
         if (msg.startOfAnimation) initGeneralPath(path);
      }

      one[0]=(int) (oWidthXY+data.getL1()*Math.cos(data.getAlpha()));
      one[1]=(int) (oHeightXY-data.getL1()*Math.sin(data.getAlpha()));

      pos1=(int) (one[0]-data.getL2()*Math.cos(data.getAlpha()+data.getBeta()));
      pos2=(int) (one[1]+data.getL2()*Math.sin(data.getAlpha()+data.getBeta()));

      if (drawPath)
         if((msg!=null && msg.isVertice) || (counter%10==0 && AnimationLoop.isAB)) path.lineTo(pos1, pos2);

      if (counter%((int) (personalDelay*AnimationLoop.delay)+1)==0) paintImmediately(0,0,getWidth(), getHeight());
         
      counter++;
   };

   public void updateDataModel(int x, int y) {AnimationLoop.data.setCoordinates(x,y);}

   public double getACoordinate(int s, int t) {
      return Math.toDegrees(AnimationLoop.data.newAlphaAngle(s, t));
   };

   public double getBCoordinate(int s, int t) {
      return Math.toDegrees(AnimationLoop.data.newBetaAngle(s,t));
   }
}
